<?php

namespace App\Http\Controllers;

use App\Models\Admin;
use App\Models\AdminLog;
use App\Models\Role;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Tymon\JWTAuth\JWTAuth;
use Zhuzhichao\IpLocationZh\Ip;
use Jenssegers\Agent\Facades\Agent;
use Carbon\Carbon;

class AuthController extends Controller
{
    protected $jwt;
    /**
     * Create a new AuthController instance.
     *
     * @return void
     */
    public function __construct(JWTAuth $JWTAuth)
    {
        $this->jwt = $JWTAuth;
        $this->middleware('auth:api', ['except' => ['login', 'refresh']]);
    }

    /**
     * Get a JWT via given credentials.
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function login(Request $request)
    {
        $input=$request->all();
        $rules=[
            'name'=>'required|between:5,12',
            'password'=>'required|between:6,12'
        ];
        $messages=[
            'name.required'=>'账号不能为空.',
            'name.between'=>'账号长度应为5-12个字符.',
            'password.required'=>'密码不能为空.',
            'password.between'=>'密码长度应为6-12个字符.',
        ];
        $validator = Validator::make($input, $rules, $messages);
        if($validator->fails()){
            return $this->error('400013',$validator->errors()->first());
        }
        if (!$token = auth('api')->attempt($request->only('name', 'password'))) {
            return $this->error('400004','账户名称和密码有误');
        }
        $user=Admin::where('name',$request->input('name'))->first();
        $ip=$request->ip();
        $address=Ip::find($ip);
        AdminLog::create([
            'ip'=>$ip,
            'address'=>$address[0].$address[1].$address[2],
            'time'=>Carbon::now()->toDateTimeString(),
            'created_at'=>Carbon::now()->toDateTimeString(),
            'updated_at'=>Carbon::now()->toDateTimeString(),
            'browser'=>Agent::browser(),//浏览器名称
            'platform'=>Agent::platform(), //操作系统
            'device'=>Agent::device(),//设备名称
            'is_phone'=>Agent::isPhone(),
            'admin_name'=>$request->input('name')
        ]);
        return $this->success($token,$user->id);
    }

    /**
     * Get the authenticated User.
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function me()
    {
        return $this->success(auth('api')->user());
    }

    /**
     * Log the user out (Invalidate the token).
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function logout()
    {
        auth('api')->logout();

        return $this->success([],'退出成功');
    }

    /**
     * Refresh a token.
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function refresh()
    {
        return $this->respondWithToken(auth('api')->refresh());
    }

    /**
     * Get the token array structure.
     *
     * @param  string $token
     *
     * @return \Illuminate\Http\JsonResponse
     */
    protected function respondWithToken($token)
    {
        return response()->json([
            'access_token' => $token,
            'token_type' => 'Bearer',
            'expires_in' => auth('api')->factory()->getTTL() * 60
        ]);
    }

    /* 管理员修改密码 */
    public function updatePassword(Request $request)
    {
        $adminId = Auth::user()->id;
        $input = $request->all();
        $admin=Admin::find($adminId);
        if (!Hash::check($input['oldpassword'], $admin->password)){
            return $this->error('400014','密码验证有误');
        }
        if (Hash::check($input['password'], $admin->password)){
            return $this->error('400014','新密码与旧密码相同');
        }
        $admin->password=Hash::make($input['password']);
        $row=$admin->save();
        if ($row) {
            Log::info('管理员:'.Auth::user()->name.'(id='.Auth::user()->id.')修改密码成功');
            Auth::logout();
            return $this->success([]);
        }
        return $this->error();
    }

    //获取当前管理员信息
    public function getCurrentAdmin(){
        $re=Admin::where('id',Auth::user()->id)->first();
        if ($re->role_ids){
            $role=Role::find($re->role_ids);
            $admin=$re;
            $admin->role->id=$role->id;
            $admin->role->name=$role->name;
            $admin->role->permissions=$role->permissions;
        }else{
            $admin=Admin::with('role:id,name,permissions')->find(Auth::user()->id);
        }

        if($admin){

            return $this->success($admin);
        }
        return $this->error();
    }

}
